/**
 * ISC License
 *
 * Copyright (c) 2018, Aleck Greenham
 *
 * Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import PropTypes from 'prop-types';
import React, { PureComponent, Component } from 'react';

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _superPropBase(object, property) {
  while (!Object.prototype.hasOwnProperty.call(object, property)) {
    object = _getPrototypeOf(object);
    if (object === null) break;
  }

  return object;
}

function _get(target, property, receiver) {
  if (typeof Reflect !== "undefined" && Reflect.get) {
    _get = Reflect.get;
  } else {
    _get = function _get(target, property, receiver) {
      var base = _superPropBase(target, property);

      if (!base) return;
      var desc = Object.getOwnPropertyDescriptor(base, property);

      if (desc.get) {
        return desc.get.call(receiver);
      }

      return desc.value;
    };
  }

  return _get(target, property, receiver || target);
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function arrayFrom(a){return Array.isArray(a)?a:a?[a]:[]}

function nop(a){return a}

function dictionaryFrom(a){var b=!(1<arguments.length&&arguments[1]!==void 0)||arguments[1],c=2<arguments.length&&arguments[2]!==void 0?arguments[2]:nop,d=3<arguments.length&&arguments[3]!==void 0?arguments[3]:{},e=function(){return "function"==typeof b?b:function(){return b}}();return arrayFrom(a).reduce(function(a,b){return a[c(b)]=e(b),a},d)}

var objectValues = (function(a){return Object.keys(a).map(function(b){return a[b]})});

var _defaultConfiguration={logLevel:"warn",defaultKeyEvent:"keydown",defaultComponent:"div",defaultTabIndex:"-1",ignoreTags:["input","select","textarea"],ignoreKeymapAndHandlerChangesByDefault:!0,ignoreEventsCondition:function c(a){var b=a.target;if(b&&b.tagName){var d=b.tagName.toLowerCase();return Configuration.option("_ignoreTagsDict")[d]||b.isContentEditable}return !1},ignoreRepeatedEventsWhenKeyHeldDown:!0,simulateMissingKeyPressEvents:!0,stopEventPropagationAfterHandling:!0,stopEventPropagationAfterIgnoring:!0,allowCombinationSubmatches:!1,customKeyCodes:{}},_configuration=_objectSpread({},_defaultConfiguration);_configuration._ignoreTagsDict=dictionaryFrom(_configuration.ignoreTags);var Configuration=function(){function a(){_classCallCheck(this,a);}return _createClass(a,null,[{key:"init",value:function e(a){var b=this,c=a.ignoreTags,d=a.customKeyCodes;c&&(a._ignoreTagsDict=dictionaryFrom(a.ignoreTags)),d&&(a._customKeyNamesDict=dictionaryFrom(objectValues(a.customKeyCodes))),-1!==["verbose","debug","info"].indexOf(a.logLevel)&&console.warn("React HotKeys: You have requested log level '".concat(a.logLevel,"' but for performance reasons, logging below severity level 'warning' is disabled in production. Please use the development build for complete logs.")),Object.keys(a).forEach(function(c){b.set(c,a[c]);});}},{key:"set",value:function c(a,b){_configuration[a]=b;}},{key:"reset",value:function b(a){_configuration[a]=_defaultConfiguration[a];}},{key:"option",value:function b(a){return _configuration[a]}}]),a}();

function backwardsCompatibleContext(a,b){var c=b.deprecatedAPI,d=c.contextTypes,e=c.childContextTypes,f=b.newAPI.contextType;if("undefined"==typeof React.createContext)a.contextTypes=d,a.childContextTypes=e,a.prototype.getChildContext=function(){return this._manager.childContext};else{var g=React.createContext(f);a.contextType=g,a.prototype._originalRender=a.prototype.render,a.prototype.render=function(){var a=this._originalRender();return a?React.createElement(g.Provider,{value:this._manager.childContext},a):null};}return a}

function isUndefined(a){return "undefined"==typeof a}

var Logger=function(){function a(){var b=this,c=0<arguments.length&&void 0!==arguments[0]?arguments[0]:"warn";_classCallCheck(this,a),_defineProperty(this,"verbose",this.noop),_defineProperty(this,"debug",this.noop),_defineProperty(this,"info",this.noop),_defineProperty(this,"warn",this.noop),_defineProperty(this,"error",this.noop),this.logLevel=this.constructor.levels[c];for(var d,e=["error","warn","info","debug","verbose"],f=function(){var a=e[g];if(b.logLevel<b.constructor.levels[a])return {v:void 0};var c=-1===["debug","verbose"].indexOf(a)?console[a]:console.log;b[a]=function(){for(var a=arguments.length,b=Array(a),d=0;d<a;d++)b[d]=arguments[d];var e=b.map(function(a){return "function"==typeof a?a():a});c.apply(void 0,_toConsumableArray(e));};},g=0;g<e.length;g++)if(d=f(),"object"===_typeof(d))return d.v}return _createClass(a,[{key:"noop",value:function a(){}}]),a}();_defineProperty(Logger,"logIcons",["\uD83D\uDCD5","\uD83D\uDCD7","\uD83D\uDCD8","\uD83D\uDCD9"]),_defineProperty(Logger,"componentIcons",["\uD83D\uDD3A","\u2B50\uFE0F","\uD83D\uDD37","\uD83D\uDD36","\u2B1B\uFE0F"]),_defineProperty(Logger,"eventIcons",["\u2764\uFE0F","\uD83D\uDC9A","\uD83D\uDC99","\uD83D\uDC9B","\uD83D\uDC9C","\uD83E\uDDE1"]),_defineProperty(Logger,"levels",{none:0,error:1,warn:2,info:3,debug:4,verbose:5});

var KeyEventType={keydown:0,keypress:1,keyup:2};

var KeyEventSequenceIndex={previous:0,current:1};

var ShiftedKeysDictionary={"`":["~"],1:["!"],2:["@","\""],3:["#","\xA3"],4:["$"],5:["%"],6:["^"],7:["&"],8:["*"],9:["("],0:[")"],"-":["_"],"=":["plus"],";":[":"],"'":["\"","@"],",":["<"],".":[">"],"/":["?"],"\\":["|"],"[":["{"],"]":["}"],"#":["~"]};

function fallbackToTransformedSelf(a,b,c){return a[b]||[1===b.length?b[c]():b]}

function resolveShiftedAlias(a){return fallbackToTransformedSelf(ShiftedKeysDictionary,a,"toUpperCase")}

function hasKey(a,b){return a.hasOwnProperty(b)}

function invertArrayDictionary(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{};return Object.keys(a).reduce(function(c,d){var e=a[d];return e.forEach(function(a){hasKey(c,a)||(c[a]=[]),c[a].push(d);}),b.includeOriginal&&(!hasKey(c,d)&&(c[d]=[]),c[d]=[].concat(_toConsumableArray(c[d]),_toConsumableArray(e))),c},{})}

var UnshiftedKeysDictionary=invertArrayDictionary(ShiftedKeysDictionary);

function resolveUnshiftedAlias(a){return fallbackToTransformedSelf(UnshiftedKeysDictionary,a,"toLowerCase")}

var KeyOSAndLayoutAliasesDictionary={};var KeyOSAndLayoutAliasesDictionary$1 = invertArrayDictionary(KeyOSAndLayoutAliasesDictionary,{includeOriginal:!0});

function isString(a){return "string"==typeof a}

function stripSuperfluousWhitespace(a){return isString(a)?a.trim().replace(/\s+/g," "):a}

var MousetrapToReactKeyNamesDictionary={tab:"Tab",capslock:"CapsLock",shift:"Shift",meta:"Meta",alt:"Alt",ctrl:"Control",space:" ",spacebar:" ",escape:"Escape",esc:"Escape",left:"ArrowLeft",right:"ArrowRight",up:"ArrowUp",down:"ArrowDown",return:"Enter",del:"Delete",command:"Meta",option:"Alt",enter:"Enter",backspace:"Backspace",ins:"Insert",pageup:"PageUp",pagedown:"PageDown",end:"End",home:"Home",contextmenu:"ContextMenu",numlock:"Clear"};

var KeyShorthandDictionary={cmd:"Meta"};

var AltShiftedKeysDictionary={"`":["`"],1:["\u2044"],2:["\u20AC"],3:["\u2039"],4:["\u203A"],5:["\uFB01"],6:["\uFB02"],7:["\u2021"],8:["\xB0"],9:["\xB7"],0:["\u201A"],"-":["\u2014"],"=":["\xB1"],a:["\xC5"],b:["\u0131"],c:["\xC7"],d:["\xCE"],e:["\xB4"],f:["\xCF"],g:["\u02DD"],h:["\xD3"],i:["\u02C6"],j:["\xD4"],k:["\uF8FF"],l:["\xD2"],m:["\xC2"],n:["\u02DC"],o:["\xD8"],p:["\u03C0"],q:["\u0152"],r:["\u2030"],s:["\xCD"],t:["\xCE"],u:["\xA8"],v:["\u25CA"],w:["\u201E"],x:["\u02DB"],y:["\xC1"],z:["\xB8"],"[":["\u201D"],"]":["\u2019"],"\\":["\xBB"],"'":["\xC6"],";":["\xDA"],",":["\xAF"],".":["\u02D8"]};

var UnaltShiftedKeysDictionary=invertArrayDictionary(AltShiftedKeysDictionary);

function resolveUnaltShiftedAlias(a){return UnaltShiftedKeysDictionary[a]||resolveUnshiftedAlias(a)}

function standardizeKeyName(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{shift:!1,alt:!1},c=a.toLowerCase(),d=MousetrapToReactKeyNamesDictionary[c]||KeyShorthandDictionary[c]||(a.match(/^f\d+$/)?a.toUpperCase():a);return b.shift?b.alt?resolveUnaltShiftedAlias(d):resolveUnshiftedAlias(d):d}

var translateToKey={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"};

var NonPrintableKeysDictionary=dictionaryFrom(objectValues(translateToKey));

function isNonPrintableKeyName(a){return !!NonPrintableKeysDictionary[a]}

function isCustomKeyName(a){return Configuration.option("_customKeyNamesDict")[a]}

function isValidKey(a){return isNonPrintableKeyName(a)||String.fromCharCode(a.charCodeAt(0))===a||isCustomKeyName(a)}var InvalidKeyNameError=function(a){function b(){var a,c;_classCallCheck(this,b);for(var d=arguments.length,e=Array(d),f=0;f<d;f++)e[f]=arguments[f];return c=_possibleConstructorReturn(this,(a=_getPrototypeOf(b)).call.apply(a,[this].concat(e))),_defineProperty(_assertThisInitialized(_assertThisInitialized(c)),"name","InvalidKeyNameError"),c}return _inherits(b,a),b}(_wrapNativeSuper(Error));

function normalizedCombinationId(a){return Object.keys(a).sort(function(c,a){return c.length===a.length?c<a?-1:c>a?1:0:a.length-c.length}).join("+")}

var KeySequenceParser=function(){function a(){_classCallCheck(this,a);}return _createClass(a,null,[{key:"parse",value:function k(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},c=stripSuperfluousWhitespace(a),d=c.split(" ");try{var e=d.slice(0,d.length-1),f=d[d.length-1],g=e.map(function(a){var c=parseCombination(a,b);return normalizedCombinationId(c)}).join(" "),h=parseCombination(f,b),i=normalizedCombinationId(h),j={id:i,keyDictionary:h,keyEventType:b.keyEventType,size:Object.keys(h).length};return {sequence:{prefix:g,size:e.length+1},combination:j}}catch(a){return {sequence:null,combination:null}}}}]),a}();function parseCombination(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{},c=-1!==a.indexOf(/[sS]hift/),d=-1!==a.indexOf(/[aA]lt/);return a.replace(/^\+|(\s|[^+]\+)\+/,"$1plus").split("+").reduce(function(a,e){var f=standardizeKeyName(e,{shift:c,alt:d});if(b.ensureValidKeys&&!isValidKey(f))throw new InvalidKeyNameError;return a[f]=!0,a},{})}

var AltedKeysDictionary={"`":["`"],1:["\xA1"],2:["\u2122"],3:["\xA3"],4:["\xA2"],5:["\u221E"],6:["\xA7"],7:["\xB6"],8:["\u2022"],9:["\xAA"],0:["\xBA"],"-":["\u2013"],"=":["\u2260"],a:["\xE5"],b:["\u222B"],c:["\xE7"],d:["\u2202"],e:["\xB4"],f:["\u0192"],g:["\xA9"],h:["\u02D9"],i:["\u02C6"],j:["\u2206"],k:["\u02DA"],l:["\xAC"],m:["\xB5"],n:["\u02DC"],o:["\xF8"],p:["\u03C0"],q:["\u0153"],r:["\xAE"],s:["\xDF"],t:["\u2020"],u:["\xA8"],v:["\u221A"],w:["\u2211"],x:["\u2248"],y:["\xA5"],z:["\u03A9"],"[":["\u201C"],"]":["\u2018"],"\\":["\xAB"],"'":["\xE6"],";":["\u2026"],",":["\u2264"],".":["\u2265"],"/":["\xF7"]};

var UnaltedKeysDictionary=invertArrayDictionary(AltedKeysDictionary);

function resolveUnaltedAlias(a){return UnaltedKeysDictionary[a]||[a]}

function resolveAltedAlias(a){return AltedKeysDictionary[a]||[a]}

function resolveAltShiftedAlias(a){return AltShiftedKeysDictionary[a]||[a]}

function isObject(a){return !Array.isArray(a)&&"object"===_typeof(a)&&null!==a}

function size(a){return isObject(a)?Object.keys(a).length:a.length}

function distinct(a){return Object.keys(dictionaryFrom(a))}

function buildShiftedKeyAliases(a,b){return a?[].concat(_toConsumableArray(resolveAltShiftedAlias(b)),_toConsumableArray(resolveUnaltShiftedAlias(b)),[b]):[].concat(_toConsumableArray(resolveShiftedAlias(b)),_toConsumableArray(resolveUnshiftedAlias(b)),[b])}function buildAltKeyAliases(a){return [].concat(_toConsumableArray(resolveAltedAlias(a)),_toConsumableArray(resolveUnaltedAlias(a)),[a])}function buildOSAndKeyboardLayoutAliases(a){var b=KeyOSAndLayoutAliasesDictionary$1[a];return b?[a].concat(_toConsumableArray(b)):[a]}function buildKeyAliasList(a,b){var c=a.Shift,d=a.Alt,e=function(){return c?buildShiftedKeyAliases(d,b):d?buildAltKeyAliases(b):buildOSAndKeyboardLayoutAliases(b)}();return distinct(e)}function buildKeyCombinationPermutations(a){return Object.keys(a).reduce(function(b,c){var d=buildKeyAliasList(a,c);return 0===size(b)?d.map(function(a){return _defineProperty({},a,!0)}):d.reduce(function(a,c){return a.concat(b.map(function(a){return _objectSpread({},a,_defineProperty({},c,!0))}))},[])},[])}var KeyCombinationSerializer=function(){function a(){_classCallCheck(this,a);}return _createClass(a,null,[{key:"serialize",value:function c(a){var b=buildKeyCombinationPermutations(a);return b.map(normalizedCombinationId)}},{key:"isValidKeySerialization",value:function b(a){return !!(0<a.length)&&!!KeySequenceParser.parse(a,{ensureValidKeys:!0}).combination}}]),a}();

function resolveKeyAlias(a){return KeyOSAndLayoutAliasesDictionary$1[a]||[a]}

function applicableAliasFunctions(a){return a.Shift?a.Alt?[resolveAltShiftedAlias,resolveUnaltShiftedAlias]:[resolveShiftedAlias,resolveUnshiftedAlias]:a.Alt?[resolveAltedAlias,resolveUnaltedAlias]:[nop$1,nop$1]}function nop$1(a){return [a]}

var KeyEventState={unseen:0,seen:1,simulated:2};

var KeyEventStateArrayManager=function(){function a(){_classCallCheck(this,a);}return _createClass(a,null,[{key:"newRecord",value:function d(a,b){var c=[KeyEventState.unseen,KeyEventState.unseen,KeyEventState.unseen];if(!isUndefined(a))for(var e=0;e<=a;e++)c[e]=b;return c}},{key:"setBit",value:function d(a,b,c){return a[b]=c,a}},{key:"clone",value:function d(a){for(var b=this.newRecord(),c=0;c<a.length;c++)b[c]=a[c];return b}}]),a}();

function isEmpty(a){return isObject(a)?0===Object.keys(a).length:!a||0===a.length}

function lazyLoadAttribute(a,b,c){return a[b]||(a[b]="function"==typeof c?c():c),a[b]}

var ModifierFlagsDictionary={Shift:["shiftKey"],Meta:["metaKey"],Control:["ctrlKey"],Alt:["altKey"]};

function stateFromEvent(a){return a.simulated?KeyEventState.simulated:KeyEventState.seen}

var KeyCombination=function(){function a(){var b=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{};_classCallCheck(this,a),this.keyStates=b,this.isEnding=!1,this._keyAliases=void 0,this._ids=void 0;}return _createClass(a,[{key:"getNormalizedKeyName",value:function d(a){var b=this.keyStates[a];if(b)return a;var c=this.keyAliases[a];return c?c:a}},{key:"hasEnded",value:function a(){return isEmpty(this.keysStillPressedDict())}},{key:"addKey",value:function c(a,b){this._setKeyState(a,[KeyEventStateArrayManager.newRecord(),KeyEventStateArrayManager.newRecord(KeyEventType.keydown,b)]);}},{key:"setKeyState",value:function g(a,b,c){var d=this._getKeyState(a);if(this.isKeyIncluded(a)){var e=KeyEventStateArrayManager.clone(d[1]),f=KeyEventStateArrayManager.clone(e);KeyEventStateArrayManager.setBit(f,b,c),this._setKeyState(a,[e,f]);}else this.addKey(a,c);b===KeyEventType.keyup&&(this.isEnding=!0);}},{key:"keysStillPressedDict",value:function b(){var a=this;return this.keys.reduce(function(b,c){return a.isKeyStillPressed(c)&&(b[c]=a._getKeyState(c)),b},{})}},{key:"isKeyIncluded",value:function b(a){return !!this._getKeyState(a)}},{key:"isKeyStillPressed",value:function b(a){return this.isEventTriggered(a,KeyEventType.keypress)&&!this.isKeyReleased(a)}},{key:"isKeyReleased",value:function b(a){return this.isEventTriggered(a,KeyEventType.keyup)}},{key:"isEventTriggered",value:function c(a,b){return this._getKeyStateType(a,KeyEventSequenceIndex.current,b)}},{key:"wasEventPreviouslyTriggered",value:function c(a,b){return this._getKeyStateType(a,KeyEventSequenceIndex.previous,b)}},{key:"isKeyPressSimulated",value:function b(a){return this.isEventTriggered(a,KeyEventType.keypress)===KeyEventState.simulated}},{key:"isKeyUpSimulated",value:function b(a){return this.isEventTriggered(a,KeyEventType.keyup)===KeyEventState.simulated}},{key:"resolveModifierFlagDiscrepancies",value:function e(a,b,c){var d=this;Object.keys(ModifierFlagsDictionary).forEach(function(e){if(b!==e||c!==KeyEventType.keyup){var f=d.isKeyStillPressed(e);ModifierFlagsDictionary[e].forEach(function(b){!1===a[b]&&f&&d.setKeyState(e,KeyEventType.keyup,stateFromEvent(a)),!0!==a[b]||f||d.setKeyState(e,KeyEventType.keydown,stateFromEvent(a));});}});}},{key:"_getKeyStateType",value:function e(a,b,c){var d=this._getKeyState(a);return d&&d[b][c]}},{key:"_getKeyState",value:function d(a){var b=this.keyStates[a];if(b)return b;var c=this.keyAliases[a];if(c)return this.keyStates[c]}},{key:"_setKeyState",value:function d(a,b){var c=this.getNormalizedKeyName(a);this.keyStates[c]=b,delete this._keyAliases,delete this._ids;}},{key:"ids",get:function b(){var a=this;return lazyLoadAttribute(this,"_ids",function(){return KeyCombinationSerializer.serialize(a.keyStates)})}},{key:"keyAliases",get:function b(){var a=this;return lazyLoadAttribute(this,"_keyAliases",function(){return buildKeyAliases(a.keyStates)})}},{key:"keys",get:function a(){return Object.keys(this.keyStates)}}]),a}();function buildKeyAliases(a){return Object.keys(a).reduce(function(b,c){return resolveKeyAlias(c).forEach(function(d){applicableAliasFunctions(a).forEach(function(a){a(d).forEach(function(a){(a!==c||c!==d)&&(b[a]=c);});});}),b},{})}

var KeyCombinationDecorator=function(){function a(b){_classCallCheck(this,a),this._keyCombination=b;}return _createClass(a,[{key:"describe",value:function a(){return this._keyCombination.ids[0]}},{key:"asKeyDictionary",value:function a(){return dictionaryFrom(this._keyCombination.keys)}},{key:"toJSON",value:function a(){return {keys:this._keyCombination.keyStates,ids:this._keyCombination.ids,keyAliases:this._keyCombination.keyAliases}}}]),a}();

var KeyCombinationIterator=function(){function a(b){_classCallCheck(this,a),this._keyCombination=b;}return _createClass(a,[{key:"any",value:function a(){return 0<this._getKeys().length}},{key:"forEachKey",value:function b(a){return this._getKeys().forEach(a)}},{key:"some",value:function b(a){return this._getKeys().some(a)}},{key:"_getKeys",value:function a(){return this._keyCombination.keys}},{key:"numberOfKeys",get:function a(){return size(this._getKeys())}}]),a}();

var KeyHistory=function(){function a(b){var c=b.maxLength,d=1<arguments.length&&void 0!==arguments[1]?arguments[1]:null;_classCallCheck(this,a),this._combinations=[],this._maxLength=c,d?this._push(d):this._push(new KeyCombination);}return _createClass(a,[{key:"getPreviousCombinations",value:function b(a){return this._combinations.slice(-(a+1),-1)}},{key:"any",value:function a(){return this._combinations.some(function(a){return new KeyCombinationIterator(a).any()})}},{key:"addKeyToCurrentCombination",value:function d(a,b,c){this._ensureInitialKeyCombination(),this.currentCombination.setKeyState(a,b,c);}},{key:"startNewKeyCombination",value:function d(a,b){this._ensureInitialKeyCombination();var c=new KeyCombination(this.currentCombination.keysStillPressedDict());c.addKey(a,b),this._push(c);}},{key:"toJSON",value:function a(){return this._combinations.map(function(a){return new KeyCombinationDecorator(a).toJSON()})}},{key:"_ensureInitialKeyCombination",value:function a(){0===this.length&&this._push(new KeyCombination);}},{key:"_push",value:function b(a){this._trimHistory(),this._combinations.push(a);}},{key:"_trimHistory",value:function a(){for(;this.length>this._maxLength;)this._shift();}},{key:"_shift",value:function a(){this._combinations.shift();}},{key:"length",get:function a(){return this._combinations.length}},{key:"currentCombination",get:function a(){return this._combinations[this.length-1]}},{key:"maxLength",set:function b(a){this._maxLength=a,this._trimHistory();}}]),a}();

var Registry=function(){function a(){_classCallCheck(this,a),this._registry={};}return _createClass(a,[{key:"get",value:function b(a){return this._registry[a]}},{key:"set",value:function c(a,b){this._registry[a]=b;}},{key:"remove",value:function b(a){delete this._registry[a];}},{key:"toJSON",value:function a(){return this._registry}}]),a}();

function without(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:[],c=2<arguments.length&&arguments[2]!==void 0?arguments[2]:{},d=dictionaryFrom(arrayFrom(b),function(a){return {value:a}});return Array.isArray(a)?a.reduce(function(a,b){return d[b]&&(c.stringifyFirst||d[b].value===b)||a.push(b),a},[]):isObject(a)?Object.keys(a).reduce(function(b,c){return d[c]||(b[c]=a[c]),b},{}):a}

var ComponentTree=function(a){function b(){var a;return _classCallCheck(this,b),a=_possibleConstructorReturn(this,_getPrototypeOf(b).call(this)),a.clearRootId(),a}return _inherits(b,a),_createClass(b,[{key:"hasRoot",value:function a(){return !this.isRootId(null)}},{key:"isRootId",value:function b(a){return a===this.rootId}},{key:"clearRootId",value:function a(){this.rootId=null;}},{key:"add",value:function d(a,c){_get(_getPrototypeOf(b.prototype),"set",this).call(this,a,{childIds:[],parentId:null,keyMap:c});}},{key:"update",value:function e(a,c){var d=_get(_getPrototypeOf(b.prototype),"get",this).call(this,a);_get(_getPrototypeOf(b.prototype),"set",this).call(this,a,_objectSpread({},d,{keyMap:c}));}},{key:"setParent",value:function c(a,b){isUndefined(b)?this.rootId=a:(this.get(a).parentId=b,this._addChildId(b,a));}},{key:"remove",value:function d(a){var c=this._getParentId(a);this._removeChildId(c,a),_get(_getPrototypeOf(b.prototype),"remove",this).call(this,a);}},{key:"_getParentId",value:function c(a){var b=this.get(a);return b&&b.parentId}},{key:"_addChildId",value:function c(a,b){this.get(a).childIds.push(b);}},{key:"_removeChildId",value:function d(a,b){var c=this.get(a);c&&(c.childIds=without(c.childIds,b));}}]),b}(Registry);

function removeAtIndex(a,b){return [].concat(_toConsumableArray(a.slice(0,b)),_toConsumableArray(a.slice(b+1)))}

var ComponentOptionsListIterator=function(){function a(b){_classCallCheck(this,a),this._list=b,this.position=-1;}return _createClass(a,[{key:"getComponent",value:function a(){return this._list.getAtPosition(this.position)}},{key:"next",value:function a(){return this.position+1<this._list.length?(this.position++,this.getComponent()):null}}]),a}();

var ComponentOptionsList=function(){function a(){_classCallCheck(this,a),this._list=[],this._idToIndex={},this.longestSequence=1,this._longestSequenceComponentId=null,this._keyMapEventRecord=KeyEventStateArrayManager.newRecord();}return _createClass(a,[{key:"add",value:function g(a,b,c,d){if(this.containsId(a))return this.update(a,b,c,d);var e=this._build(a,b,c,d);this._list.push(e);var f=this.length-1;return this._idToIndex[a]=f}},{key:"containsId",value:function b(a){return !!this.getById(a)}},{key:"getById",value:function b(a){return this.getAtPosition(this.getPositionById(a))}},{key:"getPositionById",value:function b(a){return this._idToIndex[a]}},{key:"update",value:function h(a,b,c,d){var e=this._isUpdatingComponentWithLongestSequence(a),f=this.longestSequence,g=this._build(a,b,c,d);e&&g.sequenceLength!==f&&(g.sequenceLength>f?this.longestSequence=g.sequenceLength:this._recalculateLongestSequence()),this._list[this.getPositionById(a)]=g;}},{key:"remove",value:function c(a){var b=this._isUpdatingComponentWithLongestSequence(a);this.removeAtPosition(this.getPositionById(a)),b&&this._recalculateLongestSequence();}},{key:"isRoot",value:function b(a){return this.getPositionById(a)>=this.length-1}},{key:"anyActionsForEventType",value:function b(a){return !!this._keyMapEventRecord[a]}},{key:"getAtPosition",value:function b(a){return this._list[a]}},{key:"removeAtPosition",value:function b(a){this._list=removeAtIndex(this._list,a);for(var c=a;c<this.length;)this._idToIndex[this.getAtPosition(c).componentId]=c,c++;}},{key:"toJSON",value:function a(){return this._list}},{key:"_build",value:function f(a,b,c,d){var e=this._buildActionDictionary(b,d,a);return {actions:e,handlers:c,componentId:a,options:d}}},{key:"_isUpdatingComponentWithLongestSequence",value:function b(a){return a===this._getLongestSequenceComponentId()}},{key:"_getLongestSequenceComponentId",value:function a(){return this._longestSequenceComponentId}},{key:"_recalculateLongestSequence",value:function d(){for(var e=this.iterator;e.next();){var a=e.getComponent(),b=a.longestSequence,c=a.componentId;b>this.longestSequence&&(this._longestSequenceComponentId=c,this.longestSequence=b);}}},{key:"_buildActionDictionary",value:function e(a,b,c){var d=this;return Object.keys(a).reduce(function(e,f){var g=a[f],h=function(){return isObject(g)&&hasKey(g,"sequences")?arrayFrom(g.sequences):arrayFrom(g)}();return h.forEach(function(a){var g=normalizeActionOptions(a,b),h=g.keySequence,i=g.keyEventType;d._addActionOptions(e,c,f,h,i);}),e},{})}},{key:"_addActionOptions",value:function i(a,b,c,d,e){var f=KeySequenceParser.parse(d,{keyEventType:e}),g=f.sequence,h=f.combination;g.size>this.longestSequence&&(this.longestSequence=g.size,this._longestSequenceComponentId=b),this._keyMapEventRecord[e]=KeyEventState.seen,a[c]||(a[c]=[]),a[c].push(_objectSpread({prefix:g.prefix,actionName:c,sequenceLength:g.size},h));}},{key:"iterator",get:function a(){return new ComponentOptionsListIterator(this)}},{key:"length",get:function a(){return this._list.length}}]),a}();function normalizeActionOptions(a,b){if(isObject(a)){var c=a.sequence,d=a.action;return {keySequence:c,keyEventType:isUndefined(d)?KeyEventType[b.defaultKeyEvent]:KeyEventType[d]}}return {keySequence:a,keyEventType:KeyEventType[b.defaultKeyEvent]}}

function indexFromEnd(a,b){return a[a.length-(b+1)]}

for(var KeysWithKeyUpHiddenByCmd={Enter:!0,Backspace:!0,ArrowRight:!0,ArrowLeft:!0,ArrowUp:!0,ArrowDown:!0,CapsLock:!0},i=1;13>i;i++)KeysWithKeyUpHiddenByCmd["F".concat(i)]=!0;

function keyupIsHiddenByCmd(a){return 1===a.length||hasKey(KeysWithKeyUpHiddenByCmd,a)}

var KeyCombinationMatcher=function(){function a(){_classCallCheck(this,a),this._actionConfigs={},this._order=null;}return _createClass(a,[{key:"addMatch",value:function f(a,b){if(this._includesMatcherForCombination(a.id)){var c=a.keyEventType,d=a.actionName,e=a.id;this._addHandlerToActionConfig(e,{keyEventType:c,actionName:d,handler:b});}else this._addNewActionConfig(a,b);}},{key:"findMatch",value:function l(a,b,c){var d=this;lazyLoadAttribute(this,"order",function(){return d._setOrder()});var e=!0,f=!1,g=void 0;try{for(var h,i=this._order[Symbol.iterator]();!(e=(h=i.next()).done);e=!0){var j=h.value,k=this._actionConfigs[j];if(this._matchesActionConfig(a,b,c,k))return k}}catch(a){f=!0,g=a;}finally{try{e||null==i.return||i.return();}finally{if(f)throw g}}return null}},{key:"toJSON",value:function a(){return {actionConfigs:this._actionConfigs,order:this._order}}},{key:"_matchesActionConfig",value:function h(a,b,c,d){if(!canBeMatched(a,d)||!d.events[c])return !1;var e=!1,f=Object.keys(d.keyDictionary),g=f.every(function(d){return !!a.isEventTriggered(d,c)&&(b&&b===a.getNormalizedKeyName(d)&&(e=!a.wasEventPreviouslyTriggered(d,c)),!0)});return g&&e}},{key:"_setOrder",value:function b(){var a=objectValues(this._actionConfigs).reduce(function(a,b){var c=b.id,d=b.size;return a[d]||(a[d]=[]),a[d].push(c),a},{});this._order=Object.keys(a).sort(function(c,a){return a-c}).reduce(function(b,c){return b.concat(a[c])},[]);}},{key:"_addNewActionConfig",value:function j(a,b){var c=a.prefix,d=a.sequenceLength,e=a.id,f=a.keyDictionary,g=a.size,h=a.keyEventType,i=a.actionName;this._setCombinationMatcher(e,{prefix:c,sequenceLength:d,id:e,keyDictionary:f,size:g,events:{}}),this._addHandlerToActionConfig(e,{keyEventType:h,actionName:i,handler:b});}},{key:"_addHandlerToActionConfig",value:function g(a,b){var c=b.keyEventType,d=b.actionName,e=b.handler,f=this._getCombinationMatcher(a);this._setCombinationMatcher(a,_objectSpread({},f,{events:_objectSpread({},f.events,_defineProperty({},c,{actionName:d,handler:e}))}));}},{key:"_setCombinationMatcher",value:function c(a,b){this._actionConfigs[a]=b;}},{key:"_getCombinationMatcher",value:function b(a){return this._actionConfigs[a]}},{key:"_includesMatcherForCombination",value:function b(a){return !!this._getCombinationMatcher(a)}}]),a}();function canBeMatched(a,b){var c=size(b.keyDictionary),d=new KeyCombinationIterator(a),e=b.keyDictionary.Meta&&keyUpIsBeingHidden(a);return Configuration.option("allowCombinationSubmatches")||e?d.numberOfKeys>=c:d.numberOfKeys===c}function keyUpIsBeingHidden(a){return !!a.isKeyStillPressed("Meta")&&new KeyCombinationIterator(a).some(function(a){return keyupIsHiddenByCmd(a)})}

function updateIndexCounters(a,b){for(var c=0,d=!0;d&&c<a.length;){var e=indexFromEnd(a,c),f=(e+1)%(indexFromEnd(b,c)||1);a[a.length-(c+1)]=f,d=0==f,d&&c++;}return c===a.length}var KeyHistoryMatcher=function(){function a(){_classCallCheck(this,a),this._combinationMatchers={},this._eventRecord=KeyEventStateArrayManager.newRecord();}return _createClass(a,[{key:"addMatch",value:function d(a,b){var c=this._getOrCreateCombinationMatcher(a.prefix);c.addMatch(a,b),KeyEventStateArrayManager.setBit(this._eventRecord,a.keyEventType,KeyEventState.seen),(!this.longestSequence||this.longestSequence<a.sequenceLength)&&(this.longestSequence=a.sequenceLength);}},{key:"findMatch",value:function e(a,b,c){var d=this._findCombinationMatcher(a);return d?d.findMatch(a.currentCombination,a.currentCombination.getNormalizedKeyName(b),c):null}},{key:"hasMatchesForEventType",value:function b(a){return !!this._eventRecord[a]}},{key:"toJSON",value:function b(){var a=this;return Object.keys(this._combinationMatchers).reduce(function(b,c){var d=a._combinationMatchers[c];return b[c]=d.toJSON(),b},{})}},{key:"_getOrCreateCombinationMatcher",value:function b(a){return lazyLoadAttribute(this._combinationMatchers,a,function(){return new KeyCombinationMatcher})}},{key:"_findCombinationMatcher",value:function h(a){var b=a.getPreviousCombinations(this.longestSequence-1);if(0===b.length)return this._combinationMatchers[""];var c=b.map(function(a){return a.ids}),d=c.map(function(a){return a.length}),e=Array(c.length).fill(0);do{var f=e.map(function(a,b){return c[b][a]}),g=f.join(" ");if(this._combinationMatchers[g])return this._combinationMatchers[g]}while(!updateIndexCounters(e,d));return this._combinationMatchers[""]}}]),a}();

function printComponent(a){return JSON.stringify(a,componentAttributeSerializer,4)}function componentAttributeSerializer(a,b){return "function"==typeof b?b.toString():b}

function describeKeyEventType(a){switch(parseInt(a,10)){case 0:return "keydown";case 1:return "keypress";default:return "keyup";}}

var ActionResolver=function(){function a(b,c,d){_classCallCheck(this,a),this.logger=d,this._eventStrategy=c,this._keyMapMatchers=[],this._unmatchedHandlerStatus=[],this._handlersDictionary={},this._keySequencesDictionary={};for(var g=b.iterator;g.next();){var e=g.getComponent(),f=e.handlers;this._unmatchedHandlerStatus.push([Object.keys(f).length,{}]),this._keyMapMatchers.push(new KeyHistoryMatcher);}this._componentList=b,this._componentListIterator=b.iterator;}return _createClass(a,[{key:"getKeyHistoryMatcher",value:function b(a){if(this._componentHasUnmatchedHandlers(a))for(;this._componentListIterator.next();)this._addHandlersFromComponent(),this._addActionsFromComponent();return this._getKeyHistoryMatcher(a)}},{key:"componentHasActionsBoundToEventType",value:function c(a,b){return this.getKeyHistoryMatcher(a).hasMatchesForEventType(b)}},{key:"findMatchingKeySequenceInComponent",value:function e(a,b,c,d){return this.componentHasActionsBoundToEventType(a,d)?this.getKeyHistoryMatcher(a).findMatch(b,c,d):null}},{key:"callClosestMatchingHandler",value:function i(a,b,c,d,e){for(var f,g=this,h=function(){var d=g.getKeyHistoryMatcher(e),f=g._eventStrategy.componentList.getAtPosition(e),h=f.componentId;var i=g._eventStrategy.keyHistory,j=i.currentCombination,k=g.findMatchingKeySequenceInComponent(e,i,b,c);return k?(g._handleMatchFound(j,k,c,h,a,e),{v:!0}):void(g._handleMatchNotFound(j,e,c,h),e++)};e<=d;)if(f=h(),"object"===_typeof(f))return f.v}},{key:"_handleMatchNotFound",value:function f(a,b,c,d){var e=new KeyCombinationDecorator(a);if(this.componentHasActionsBoundToEventType(b,c));}},{key:"_handleMatchFound",value:function i(a,b,c,d,e,f){var g=new KeyCombinationDecorator(a),h=b.events[c];if(Configuration.option("allowCombinationSubmatches")){var j=KeyCombinationSerializer.serialize(b.keyDictionary);}h.handler(e),Configuration.option("stopEventPropagationAfterHandling")&&this._eventStrategy.stopEventPropagation(e,f);}},{key:"_getKeyHistoryMatcher",value:function b(a){return this._keyMapMatchers[a]}},{key:"_addActionsFromComponent",value:function d(){var a=this,b=this._componentListIterator.getComponent(),c=b.actions;Object.keys(c).forEach(function(b){a._buildActionMatchers(b,c[b]);});}},{key:"_buildActionMatchers",value:function e(a,b){var c=this,d=this._positionsOfComponentsWithHandlersFor(a);return !!d&&(this._registerActionWithMatcher(d,a,b),d.forEach(function(b){var d=c._getUnmatchedHandlerStatus(b),e=d[1];e[a]||(e[a]=!0,d[0]--);}),!0)}},{key:"_positionsOfComponentsWithHandlersFor",value:function b(a){return this._handlersDictionary[a]}},{key:"_registerActionWithMatcher",value:function h(a,b,c){var d=this,e=a[0],f=this._componentList.getAtPosition(e).handlers[b],g=this._getKeyHistoryMatcher(e);return c.forEach(function(a){var b=a.prefix,c=a.id,h=a.keyEventType,i=[b,c].join(" ");return !d._isClosestHandlerAlreadyFoundFor(i,h)&&void(g.addMatch(a,f),d._recordHandlerFoundFor(i,[e,h]))}),!0}},{key:"_addHandlersFromComponent",value:function d(){var a=this,b=this._componentListIterator.getComponent(),c=b.handlers;Object.keys(c).forEach(function(b){a._addHandler(b);});}},{key:"_addHandler",value:function b(a){lazyLoadAttribute(this._handlersDictionary,a,[]),this._handlersDictionary[a].push(this._componentListIterator.position);}},{key:"_recordHandlerFoundFor",value:function c(a,b){lazyLoadAttribute(this._keySequencesDictionary,a,[]),this._keySequencesDictionary[a].push(b);}},{key:"_componentHasUnmatchedHandlers",value:function b(a){return 0<this._getUnmatchedHandlerStatus(a)[0]}},{key:"_getUnmatchedHandlerStatus",value:function b(a){return this._unmatchedHandlerStatus[a]}},{key:"_isClosestHandlerAlreadyFoundFor",value:function d(a,b){var c=this._keySequencesDictionary[a]||[];return c.some(function(a){return a[1]===b})}}]),a}();

var AbstractKeyEventStrategy=function(){function a(){var c=1<arguments.length?arguments[1]:void 0;_classCallCheck(this,a),this.logger=null,this.componentId=-1,this.keyEventManager=c,this.componentTree=new ComponentTree,this._simulator=null,this._reset(),this.resetKeyHistory();}return _createClass(a,[{key:"_reset",value:function a(){this.componentList=new ComponentOptionsList,this._actionResolver=null;}},{key:"_recalculate",value:function a(){this._actionResolver=null,this.keyHistory.maxLength=this.componentList.longestSequence;}},{key:"resetKeyHistory",value:function b(){var a=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{};this._simulator&&this._simulator.clear(),this._keyHistory=this.keyHistory.any()&&!a.force?new KeyHistory({maxLength:this.componentList.longestSequence},new KeyCombination(this)):this._newKeyHistory();}},{key:"_newKeyHistory",value:function a(){return new KeyHistory({maxLength:this.componentList.longestSequence})}},{key:"registerKeyMap",value:function c(a){return this.componentId+=1,this.componentTree.add(this.componentId,a),this.componentId}},{key:"reregisterKeyMap",value:function c(a,b){this.componentTree.update(a,b);}},{key:"registerComponentMount",value:function d(a,b){this.componentTree.setParent(a,b),void 0;}},{key:"deregisterKeyMap",value:function c(a){this.componentTree.remove(a),this.componentTree.isRootId(a)&&this.componentTree.clearRootId();}},{key:"_addComponent",value:function f(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:{},e=4<arguments.length?arguments[4]:void 0;this.componentList.add(a,b,c,e),this._recalculate(),this.logger.logComponentOptions(a,this.componentList.getById(a));}},{key:"_updateComponent",value:function e(a,b,c,d){this.componentList.update(a,b,c,d),this._recalculate(),this.logger.logComponentOptions(a,this.componentList.getById(a));}},{key:"_describeCurrentCombination",value:function b(){var a=new KeyCombinationDecorator(this.currentCombination);return a.describe()}},{key:"_recordKeyDown",value:function f(a,b,c){var d=stateFromEvent(a),e=this.currentCombination;e.isKeyIncluded(b)||e.isEnding?this._startAndLogNewKeyCombination(c,b,d):this._recordNewKeyInCombination(b,KeyEventType.keydown,d,c);}},{key:"_startAndLogNewKeyCombination",value:function d(a,b,c){this.keyHistory.startNewKeyCombination(b,c),this.logger.logKeyHistory(this.keyHistory,a);}},{key:"_recordNewKeyInCombination",value:function e(a,b,c,d){this.keyHistory.addKeyToCurrentCombination(a,b,c),this.logger.logKeyHistory(this.keyHistory,d);}},{key:"_isIgnoringRepeatedEvent",value:function e(a,b,c,d){return !!(a.repeat&&Configuration.option("ignoreRepeatedEventsWhenKeyHeldDown"))&&(this.logger.logIgnoredKeyEvent(a,b,c,"it was a repeated event",d),!0)}},{key:"keyHistory",get:function b(){var a=this;return lazyLoadAttribute(this,"_keyHistory",function(){return a._newKeyHistory()})}},{key:"actionResolver",get:function b(){var a=this;return lazyLoadAttribute(this,"_actionResolver",function(){return new ActionResolver(a.componentList,a,a.logger)})}},{key:"currentCombination",get:function a(){return this.keyHistory.currentCombination}}]),a}();

function getEventCharCode(a){var b,c=a.keyCode;return "charCode"in a?(b=a.charCode,0===b&&13===c&&(b=13)):b=c,10===b&&(b=13),32<=b||13===b?b:0}

var normalizeKey={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"};function reactsGetEventKey(a){if(a.key){var b=normalizeKey[a.key]||a.key;if("Unidentified"!==b)return b}if("keypress"===a.type){var c=getEventCharCode(a);return 13===c?"Enter":String.fromCharCode(c)}return "keydown"===a.type||"keyup"===a.type?translateToKey[a.keyCode]||"Unidentified":""}

function keyNameFromEvent(a){var b=Configuration.option("customKeyCodes"),c=a.keyCode||a.charCode;return hasKey(b,c)?b[c]:a.nativeEvent?a.key:reactsGetEventKey(a)}function getKeyName(a){var b=keyNameFromEvent(a);return "+"===b?"plus":b}

function isCmdKey(a){return "Meta"===a}

function describeKeyEvent(a,b,c){var d="'".concat(b,"' ").concat(describeKeyEventType(c));return a.simulated?"(simulated) ".concat(d):d}

var EventResponse={unseen:0,ignored:1,seen:2,recorded:3,handled:4};

var KeyEventCounter=function(){function a(){_classCallCheck(this,a);}return _createClass(a,null,[{key:"incrementId",value:function a(){this._id=this.id+1;}},{key:"id",get:function a(){return lazyLoadAttribute(this,"_id",0),this._id}}]),a}();

var EventPropagator=function(){function a(b){_classCallCheck(this,a),this._componentList=b,this._previousPropagation=null,this._reset();}return _createClass(a,[{key:"_reset",value:function a(){this.previousPosition=-1,this._position=-1,this._actionHandled=!1,this._ignoreEvent=!1,this._observeIgnoredEvents=!1,this._stopping=!1,this._componentId=null,this._key=null,this._type=null;}},{key:"isFirstPropagationStep",value:function b(){var a=this.previousPosition;return -1===a||a>=this._position}},{key:"isForKey",value:function b(a){return this._key===a}},{key:"isForEventType",value:function b(a){return this._type===a}},{key:"startNewPropagationStep",value:function e(a,b,c,d){return this._position=this._componentList.getPositionById(a),this._componentId=a,this.isFirstPropagationStep()&&(KeyEventCounter.incrementId(),this._key=b.key,this._type=d),!(b.repeat&&Configuration.option("ignoreRepeatedEventsWhenKeyHeldDown"))||(this.ignoreEvent(b),!1)}},{key:"finishPropagationStep",value:function a(){this.isStopped()||this._componentList.isRoot(this._componentId)?(this._previousPropagation=this._clone(),this._reset()):this.previousPosition=this._position;}},{key:"observeIgnoredEvents",value:function a(){this._observeIgnoredEvents=!0;}},{key:"ignoreEvent",value:function b(a){return this._ignoreEvent=!0,!!(this.isIgnoringEvent()&&Configuration.option("stopEventPropagationAfterIgnoring"))&&(this.stop(a),this.finishPropagationStep(),!0)}},{key:"isIgnoringEvent",value:function a(){return !this._observeIgnoredEvents&&this._ignoreEvent}},{key:"isStopped",value:function a(){return this._stopping}},{key:"stop",value:function b(a){return !this.isStopped()&&(this._stopping=!0,a.simulated||a.stopPropagation(),!0)}},{key:"isPendingPropagation",value:function b(){var a=this.previousPosition;return -1!==a&&a+1<this._position}},{key:"isHandled",value:function a(){return this._actionHandled}},{key:"setHandled",value:function a(){this._actionHandled=!0;}},{key:"_clone",value:function e(){var b=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{},c=b.copyState,d=new a(this._componentList);return d.logger=this.logger,(void 0===c||c)&&Object.assign(d,this),d}},{key:"previousPropagation",get:function b(){var a=this;return lazyLoadAttribute(this,"_previousPropagation",function(){return a._clone({copyState:!1})})}}]),a}();

function hasKeyPressEvent(a){return !isNonPrintableKeyName(a)}

var AbstractKeyEventSimulator=function(){function a(b){_classCallCheck(this,a),this._keyEventStrategy=b,this.clear();}return _createClass(a,[{key:"clear",value:function a(){this.keypressEventsToSimulate=[],this.keyupEventsToSimulate=[];}},{key:"cloneAndMergeEvent",value:function d(a,b){var c=Object.keys(ModifierFlagsDictionary).reduce(function(b,c){return b[c]=a[c],b},{});return _objectSpread({},c,b)}},{key:"_shouldSimulate",value:function e(a,b){var c=hasKeyPressEvent(b),d=this._keyEventStrategy.currentCombination;return a===KeyEventType.keypress?!c||c&&d.isKeyStillPressed("Meta"):a===KeyEventType.keyup&&keyupIsHiddenByCmd(b)&&d.isKeyReleased("Meta")}}]),a}();

var FocusOnlyKeyEventSimulator=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"handleKeyPressSimulation",value:function b(a){this._handleEventSimulation("keypressEventsToSimulate","simulatePendingKeyPressEvents",_objectSpread({eventType:KeyEventType.keypress},a));}},{key:"handleKeyUpSimulation",value:function b(a){this._handleEventSimulation("keyupEventsToSimulate","simulatePendingKeyUpEvents",_objectSpread({eventType:KeyEventType.keyup},a));}},{key:"_handleEventSimulation",value:function j(a,b,c){var d=c.event,e=c.eventType,f=c.key,g=c.focusTreeId,h=c.componentId,i=c.options;if(this._shouldSimulate(e,f)&&Configuration.option("simulateMissingKeyPressEvents")){var k=this.cloneAndMergeEvent(d,{key:f,simulated:!0});this[a].push({event:k,focusTreeId:g,componentId:h,options:i});}(this._keyEventStrategy.componentList.isRoot(h)||this._keyEventStrategy.eventPropagator.isStopped())&&this._shouldSimulateEventsImmediately()&&this._keyEventStrategy[b]();}},{key:"simulatePendingKeyPressEvents",value:function a(){this._simulatePendingKeyEvents("keypressEventsToSimulate","handleKeyPress");}},{key:"simulatePendingKeyUpEvents",value:function a(){this._simulatePendingKeyEvents("keyupEventsToSimulate","handleKeyUp");}},{key:"_simulatePendingKeyEvents",value:function d(a,b){var c=this;0<this[a].length&&KeyEventCounter.incrementId(),this[a].forEach(function(a){var d=a.event,e=a.focusTreeId,f=a.componentId,g=a.options;c._keyEventStrategy[b](d,e,f,g);}),this[a]=[];}},{key:"_shouldSimulateEventsImmediately",value:function a(){return this._keyEventStrategy.shouldSimulateEventsImmediately()}}]),b}(AbstractKeyEventSimulator);

var FocusTree=function(){function a(){_classCallCheck(this,a),this.id=0;}return _createClass(a,[{key:"isNewerThan",value:function b(a){return this.id!==a}},{key:"new",value:function a(){this.id+=1;}}]),a}();

var EventStrategyLogger=function(a){function b(a,c){var d;return _classCallCheck(this,b),d=_possibleConstructorReturn(this,_getPrototypeOf(b).call(this,a)),d._eventStrategy=c,d}return _inherits(b,a),_createClass(b,[{key:"nonKeyEventPrefix",value:function c(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{};return this.keyEventPrefix(a,_objectSpread({},b,{eventId:!1}))}},{key:"logComponentOptions",value:function c(a,b){this.verbose(this.nonKeyEventPrefix(a),"New component options:\n",function(){return printComponent(b)});}},{key:"logKeyHistory",value:function c(a,b){this.verbose(this.keyEventPrefix(b),function(){return "Key history: ".concat(printComponent(a.toJSON()),".")});}}]),b}(Logger);

var FocusOnlyLogger=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"keyEventPrefix",value:function i(a){var c=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},d=_get(_getPrototypeOf(b.prototype),"constructor",this).logIcons,e=_get(_getPrototypeOf(b.prototype),"constructor",this).eventIcons,f=_get(_getPrototypeOf(b.prototype),"constructor",this).componentIcons,g="HotKeys (";if(!1!==c.focusTreeId){var j=isUndefined(c.focusTreeId)?this._eventStrategy.focusTreeId:c.focusTreeId;g+="F".concat(j).concat(d[j%d.length],"-");}g+="C".concat(a).concat(f[a%f.length]);var h=this._eventStrategy.componentList.getPositionById(a);if(isUndefined(h)||(g+="-P".concat(h).concat(f[h%f.length])),!1!==c.eventId){var k=isUndefined(c.eventId)?KeyEventCounter.id:c.eventId;g+="-E".concat(k).concat(e[k%e.length]);}return "".concat(g,"):")}},{key:"logIgnoredKeyEvent",value:function f(a,b,c,d,e){this.logIgnoredEvent(describeKeyEvent(a,b,c),d,e);}},{key:"logIgnoredEvent",value:function d(a,b,c){this.debug(this.keyEventPrefix(c),"Ignored ".concat(a," because ").concat(b,"."));}},{key:"logAlreadySimulatedEvent",value:function e(a,b,c,d){this.logIgnoredKeyEvent(a,b,c,"it was not expected, and has already been simulated",d);}}]),b}(EventStrategyLogger);

var FocusOnlyKeyEventStrategy=function(a){function b(){var a,c=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{},d=1<arguments.length?arguments[1]:void 0;return _classCallCheck(this,b),a=_possibleConstructorReturn(this,_getPrototypeOf(b).call(this,c,d)),a.logger=new FocusOnlyLogger(c.logLevel||"warn",_assertThisInitialized(_assertThisInitialized(a))),a.eventPropagator.logger=a.logger,a.focusTree=new FocusTree,a._simulator=new FocusOnlyKeyEventSimulator(_assertThisInitialized(_assertThisInitialized(a))),a}return _inherits(b,a),_createClass(b,[{key:"_reset",value:function a(){_get(_getPrototypeOf(b.prototype),"_reset",this).call(this),this._simulator&&this._simulator.clear(),this.focusTree&&this.focusTree.new(),this.eventPropagator=new EventPropagator(this.componentList),this.eventPropagator.logger=this.logger;}},{key:"enableHotKeys",value:function e(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:{},d=3<arguments.length?arguments[3]:void 0;if(this.resetOnNextFocus&&(this._reset(),this.resetOnNextFocus=!1),!this.componentList.containsId(a))return this._addComponent(a,b,c,"Focused",d),this.focusTree.id}},{key:"updateEnabledHotKeys",value:function f(a,b){var c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:{},d=3<arguments.length&&void 0!==arguments[3]?arguments[3]:{},e=4<arguments.length?arguments[4]:void 0;this.focusTree.isNewerThan(a)||!this.componentList.containsId(b)||this._updateComponent(b,c,d,e);}},{key:"disableHotKeys",value:function d(a,b){this.resetOnNextFocus=!0;var c=this.eventPropagator.isPendingPropagation();return c}},{key:"handleKeyDown",value:function f(a,b,c){var d=3<arguments.length&&void 0!==arguments[3]?arguments[3]:{},e=getKeyName(a);if(this.focusTree.isNewerThan(b))return this.logger.logIgnoredKeyEvent(a,e,KeyEventType.keydown,"it had an old focus tree id: ".concat(b),c),this.eventPropagator.ignoreEvent(a),!0;if(this._isIgnoringRepeatedEvent(a,e,KeyEventType.keydown,c))return !1;if(this.eventPropagator.startNewPropagationStep(c,a,e,KeyEventType.keydown)){var g=this._howToHandleKeyEvent(a,b,c,e,d,KeyEventType.keydown);g===EventResponse.handled&&(this._recordKeyDown(a,e,c),this._callHandlerIfActionNotHandled(a,e,KeyEventType.keydown,c,b)),this._simulator.handleKeyPressSimulation({event:a,key:e,focusTreeId:b,componentId:c,options:d}),this.eventPropagator.finishPropagationStep();}return !1}},{key:"_howToHandleKeyEvent",value:function g(a,b,c,d,e,f){if(this.eventPropagator.isFirstPropagationStep()){if(e.ignoreEventsCondition(a)&&this.eventPropagator.ignoreEvent(a))return this._eventIsToBeIgnored(a,c,d,f);this.currentCombination.resolveModifierFlagDiscrepancies(a,d,f);}else if(this.eventPropagator.isIgnoringEvent())return this._eventIsToBeIgnored(a,c,d,f);return EventResponse.handled}},{key:"_eventIsToBeIgnored",value:function e(a,b,c,d){return this.logger.logIgnoredKeyEvent(a,c,d,"ignoreEventsFilter rejected it",b),EventResponse.ignored}},{key:"handleKeyPress",value:function g(a,b,c,d){var e=getKeyName(a);if(this._isIgnoringRepeatedEvent(a,e,KeyEventType.keypress,c))return !1;if(this.currentCombination.isKeyPressSimulated(e))return this._ignoreAlreadySimulatedEvent(a,e,KeyEventType.keypress,c),!1;var f=this.focusTree.isNewerThan(b);if(this.eventPropagator.startNewPropagationStep(c,a,e,KeyEventType.keypress)){var h=this._howToHandleKeyEvent(a,b,c,e,d,KeyEventType.keypress);this.eventPropagator.isFirstPropagationStep(c)&&this.currentCombination.isKeyIncluded(e)&&this._recordNewKeyInCombination(e,KeyEventType.keypress,stateFromEvent(a),c),h===EventResponse.handled&&this._callHandlerIfActionNotHandled(a,e,KeyEventType.keypress,c,b),this.eventPropagator.finishPropagationStep();}return f}},{key:"handleKeyUp",value:function h(a,b,c,d){var e=getKeyName(a);if(this.currentCombination.isKeyUpSimulated(e))return this._ignoreAlreadySimulatedEvent(a,e,KeyEventType.keyup,c),!1;var f=this.focusTree.isNewerThan(b),g=this.eventPropagator;if(g.startNewPropagationStep(c,a,e,KeyEventType.keyup)){var i=this._howToHandleKeyEvent(a,b,c,e,d,KeyEventType.keyup);g.isFirstPropagationStep(c)&&this.currentCombination.isKeyIncluded(e)&&this._recordNewKeyInCombination(e,KeyEventType.keyup,stateFromEvent(a),c),i===EventResponse.handled&&this._callHandlerIfActionNotHandled(a,e,KeyEventType.keyup,c,b),this._simulateKeyUpEventsHiddenByCmd(a,e,b,c,d),g.finishPropagationStep();}return f}},{key:"_ignoreAlreadySimulatedEvent",value:function e(a,b,c,d){this.logger.logAlreadySimulatedEvent(a,b,c,d),this.eventPropagator.ignoreEvent(a);}},{key:"closeHangingKeyCombination",value:function d(a,b){var c=this.currentCombination;c.isKeyIncluded(a)&&!c.isEventTriggered(a,b)&&c.setKeyState(a,b,KeyEventState.simulated);}},{key:"_simulateKeyUpEventsHiddenByCmd",value:function g(a,b,c,d,e){var f=this;if(isCmdKey(b)){var h=new KeyCombinationIterator(this.currentCombination);h.forEachKey(function(b){isCmdKey(b)||f._simulator.handleKeyUpSimulation({event:a,key:b,focusTreeId:c,componentId:d,options:e});});}}},{key:"stopEventPropagation",value:function c(a,b){this.eventPropagator.stop(a)&&void 0;}},{key:"simulatePendingKeyPressEvents",value:function a(){this._simulator.simulatePendingKeyPressEvents();}},{key:"simulatePendingKeyUpEvents",value:function a(){this._simulator.simulatePendingKeyUpEvents();}},{key:"shouldSimulateEventsImmediately",value:function a(){return !this.keyEventManager.isGlobalListenersBound()}},{key:"_callHandlerIfActionNotHandled",value:function k(a,b,c,d,e){var f=describeKeyEventType(c),g=this._describeCurrentCombination();if(!this.componentList.anyActionsForEventType(c))return void this.logger.logIgnoredEvent("'".concat(g,"' ").concat(f),"it doesn't have any ".concat(f," handlers"),d);if(this.eventPropagator.isHandled())this.logger.logIgnoredEvent("'".concat(g,"' ").concat(f),"it has already been handled",d);else{var h=this.eventPropagator.previousPosition,i=this.componentList.getPositionById(d),j=this.actionResolver.callClosestMatchingHandler(a,b,c,i,-1===h?0:h);j&&this.eventPropagator.setHandled();}}}]),b}(AbstractKeyEventStrategy);

function findableCollectionContains(a,b,c){return c.stringifyFirst?!isUndefined(a.find(function(a){return a.toString()===b.toString()})):-1!==a.indexOf(b)}function nonCollectionContains(a,b,c){return c.stringifyFirst?a.toString()===b.toString():a===b}function contains(a,b){var c=2<arguments.length&&arguments[2]!==void 0?arguments[2]:{};return Array.isArray(a)||isString(a)?findableCollectionContains(a,b,c):isObject(a)?hasKey(a,b):nonCollectionContains(a,b,c)}

var GlobalKeyEventSimulator=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"handleKeyPressSimulation",value:function b(a){this._handleEventSimulation("handleKeyPress",_objectSpread({eventType:KeyEventType.keypress},a));}},{key:"handleKeyUpSimulation",value:function b(a){this._handleEventSimulation("handleKeyUp",_objectSpread({eventType:KeyEventType.keyup},a));}},{key:"_handleEventSimulation",value:function f(a,b){var c=b.event,d=b.eventType,e=b.key;if(this._shouldSimulate(d,e)&&Configuration.option("simulateMissingKeyPressEvents")){var g=this.cloneAndMergeEvent(c,{key:e,simulated:!0});this._keyEventStrategy[a](g);}}}]),b}(AbstractKeyEventSimulator);

function capitalize(a){return a.replace(/\b\w/g,function(a){return a.toUpperCase()})}

function normalizeEventName(a){return "".concat(capitalize(a.slice(0,3))).concat(capitalize(a.slice(3)))}

var GlobalEventListenerAdaptor=function(){function a(b,c){var d=c.logger;_classCallCheck(this,a),this._listenersBound=!1,this.logger=d,this._eventStrategy=b;}return _createClass(a,[{key:"isListenersBound",value:function a(){return this._listenersBound}},{key:"unbindListeners",value:function b(){var a=this;objectValues(KeyEventType).forEach(function(b){var c=describeKeyEventType(b);delete document["on".concat(c)],a._logHandlerStateChange("Removed",c);}),this._listenersBound=!1;}},{key:"bindListeners",value:function b(){var a=this;objectValues(KeyEventType).forEach(function(b){var c=describeKeyEventType(b);document["on".concat(c)]=function(b){a._eventStrategy["handle".concat(normalizeEventName(c))](b);},a._logHandlerStateChange("Bound",c);}),this._listenersBound=!0;}},{key:"_logHandlerStateChange",value:function c(a,b){}}]),a}();

var GlobalLogger=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"keyEventPrefix",value:function h(a){var c=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},d=_get(_getPrototypeOf(b.prototype),"constructor",this).eventIcons,e=_get(_getPrototypeOf(b.prototype),"constructor",this).componentIcons,f="HotKeys (GLOBAL";if(!isUndefined(a))return "".concat(f,"-C").concat(a).concat(e[a%e.length],"):");if(!1===c.eventId)return "".concat(f,"):");var g=isUndefined(c.eventId)?KeyEventCounter.id:c.eventId;return "".concat(f,"-E").concat(g).concat(d[g%d.length],"):")}},{key:"logIgnoredKeyEvent",value:function f(a,b,c,d,e){this.logIgnoredEvent(describeKeyEvent(a,b,c),d,e);}},{key:"logIgnoredEvent",value:function d(a,b,c){this.debug(this.keyEventPrefix(c),"Ignored ".concat(a," because ").concat(b,"."));}},{key:"logEventRejectedByFilter",value:function e(a,b,c,d){this.logIgnoredKeyEvent(a,b,c,"ignoreEventsFilter rejected it",d);}},{key:"logEventAlreadySimulated",value:function e(a,b,c,d){this.logIgnoredKeyEvent(a,b,c,"it was not expected, and has already been simulated",d);}}]),b}(EventStrategyLogger);

var GlobalKeyEventStrategy=function(a){function b(){var a,c=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{},d=1<arguments.length?arguments[1]:void 0;return _classCallCheck(this,b),a=_possibleConstructorReturn(this,_getPrototypeOf(b).call(this,c,d)),a.logger=new GlobalLogger(c.logLevel||"warn",_assertThisInitialized(_assertThisInitialized(a))),a.eventOptions={ignoreEventsCondition:Configuration.option("ignoreEventsCondition")},a.listeners=new Registry,a._simulator=new GlobalKeyEventSimulator(_assertThisInitialized(_assertThisInitialized(a))),a._listenerAdaptor=new GlobalEventListenerAdaptor(_assertThisInitialized(_assertThisInitialized(a)),{logger:a.logger}),a}return _inherits(b,a),_createClass(b,[{key:"enableHotKeys",value:function f(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:{},d=3<arguments.length?arguments[3]:void 0,e=4<arguments.length?arguments[4]:void 0;this.eventOptions=e,this._addComponent(a,b,c,"Mounted",d);}},{key:"updateEnabledHotKeys",value:function f(a){var b=1<arguments.length&&void 0!==arguments[1]?arguments[1]:{},c=2<arguments.length&&void 0!==arguments[2]?arguments[2]:{},d=3<arguments.length?arguments[3]:void 0,e=4<arguments.length?arguments[4]:void 0;this.eventOptions=e,this._updateComponent(a,b,c,d);}},{key:"disableHotKeys",value:function b(a){this.componentList.remove(a),this._recalculate();}},{key:"_recalculate",value:function a(){_get(_getPrototypeOf(b.prototype),"_recalculate",this).call(this),this._updateDocumentHandlers();}},{key:"_updateDocumentHandlers",value:function c(){var a=this._shouldListenersBeBound(),b=this.isListenersBound();!b&&a?this._listenerAdaptor.bindListeners():b&&!a&&this._listenerAdaptor.unbindListeners();}},{key:"isListenersBound",value:function a(){return this._listenerAdaptor.isListenersBound()}},{key:"_shouldListenersBeBound",value:function a(){return 0!==this.componentList.length||this.listeners.get("keyCombination")}},{key:"handleKeyDown",value:function d(a){var b=getKeyName(a);if(!this._isIgnoringRepeatedEvent(a,b,KeyEventType.keydown)){this.currentCombination.resolveModifierFlagDiscrepancies(a,b,KeyEventType.keydown);var c=this._howReactAppRespondedTo(a,b,KeyEventType.keydown);return c===EventResponse.unseen&&this.eventOptions.ignoreEventsCondition(a)?void this.logger.logEventRejectedByFilter(a,b,KeyEventType.keydown):void(c!==EventResponse.ignored&&this._recordKeyDown(a,b),this._callHandlerIfNeeded(c,a,b,KeyEventType.keydown),this.keyEventManager.simulatePendingKeyPressEvents(),this._simulator.handleKeyPressSimulation({event:a,key:b}))}}},{key:"_howReactAppRespondedTo",value:function e(a,b,c){var d=this.keyEventManager.reactAppHistoryWithEvent(b,c);return d===EventResponse.handled?this.logger.logIgnoredKeyEvent(a,b,c,"React app has already handled it"):d===EventResponse.ignored?this.logger.logIgnoredKeyEvent(a,b,c,"React app has declared it should be ignored"):d===EventResponse.seen?void 0:(KeyEventCounter.incrementId(),void 0),d}},{key:"handleKeyPress",value:function d(a){var b=getKeyName(a);if(!this._isIgnoringRepeatedEvent(a,b,KeyEventType.keypress)){if(this.currentCombination.isKeyPressSimulated(b))return void this.logger.logEventAlreadySimulated(a,b,KeyEventType.keypress);var c=this._howReactAppRespondedTo(a,b,KeyEventType.keypress);return this.currentCombination.isKeyIncluded(b)&&this._recordNewKeyInCombination(b,KeyEventType.keypress,stateFromEvent(a)),c===EventResponse.unseen&&(this.keyEventManager.closeHangingKeyCombination(b,KeyEventType.keypress),this.eventOptions.ignoreEventsCondition(a))?void this.logger.logEventRejectedByFilter(a,b,KeyEventType.keypress):void this._callHandlerIfNeeded(c,a,b,KeyEventType.keypress)}}},{key:"_callHandlerIfNeeded",value:function e(a,b,c,d){contains([EventResponse.ignored,EventResponse.handled],a)||this._callHandlerIfExists(b,c,d);}},{key:"handleKeyUp",value:function d(a){var b=getKeyName(a);if(this.currentCombination.isKeyUpSimulated(b))return void this.logger.logEventAlreadySimulated(a,b,KeyEventType.keyup);var c=this._howReactAppRespondedTo(a,b,KeyEventType.keyup);this.currentCombination.isKeyIncluded(b)&&this._recordNewKeyInCombination(b,KeyEventType.keyup,stateFromEvent(a)),c===EventResponse.unseen?(this.keyEventManager.closeHangingKeyCombination(b,KeyEventType.keyup),this.eventOptions.ignoreEventsCondition(a)?this.logger.logIgnoredKeyEvent(a,b,KeyEventType.keyup,"ignoreEventsFilter rejected it"):this._callHandlerIfNeeded(c,a,b,KeyEventType.keyup)):this._callHandlerIfNeeded(c,a,b,KeyEventType.keyup),this._simulateKeyUpEventsHiddenByCmd(a,b),this._callKeyCombinationCallbackIfDefined();}},{key:"_simulateKeyUpEventsHiddenByCmd",value:function d(a,b){var c=this;if(isCmdKey(b)){this.keyEventManager.simulatePendingKeyUpEvents();var e=new KeyCombinationIterator(this.currentCombination);e.forEachKey(function(b){isCmdKey(b)||c._simulator.handleKeyUpSimulation({event:a,key:b});});}}},{key:"_callKeyCombinationCallbackIfDefined",value:function a(){if(this.listeners.get("keyCombination")&&this.currentCombination.hasEnded()){var b=new KeyCombinationDecorator(this.currentCombination);this.listeners.get("keyCombination")({keys:b.asKeyDictionary(),id:b.describe()});}}},{key:"_callHandlerIfExists",value:function f(a,b,c){var d=describeKeyEventType(c),e=this._describeCurrentCombination();return this.componentList.anyActionsForEventType(c)?void(this._callClosestMatchingHandler(a,b,c)):void this.logger.logIgnoredEvent("'".concat(e,"' ").concat(d),"it doesn't have any ".concat(d," handlers"))}},{key:"_callClosestMatchingHandler",value:function d(a,b,c){for(var e,f=this.componentList.iterator;f.next();)if(e=this.actionResolver.callClosestMatchingHandler(a,b,c,f.position,0),e)return void void 0}},{key:"stopEventPropagation",value:function c(a,b){a.simulated||a.stopPropagation();}},{key:"addKeyCombinationListener",value:function d(a){var b=this,c=function(){b.listeners.remove("keyCombination");};return this.listeners.set("keyCombination",function(b){a(b),c();}),this._updateDocumentHandlers(),c}}]),b}(AbstractKeyEventStrategy);

function copyAttributes(a,b,c){return c.forEach(function(c){hasKey(a,c)&&(b[c]=a[c]);}),b}

var SEQUENCE_ATTRIBUTES=["sequence","action"],KEYMAP_ATTRIBUTES=["name","description","group"];function createSequenceFromConfig(a){return arrayFrom(a).map(function(a){return isObject(a)?copyAttributes(a,{},SEQUENCE_ATTRIBUTES):{sequence:a}})}function normalizeActionDefinition(a,b,c){var d=a[b];c[b]={},isObject(d)?hasKey(d,"sequences")?(copyAttributes(d,c[b],KEYMAP_ATTRIBUTES),c[b].sequences=createSequenceFromConfig(d.sequences)):(copyAttributes(d,c[b],KEYMAP_ATTRIBUTES),c[b].sequences=[copyAttributes(d,{},SEQUENCE_ATTRIBUTES)]):c[b].sequences=createSequenceFromConfig(d);}var ApplicationKeyMapBuilder=function(){function a(b){_classCallCheck(this,a),this._componentTree=b;}return _createClass(a,[{key:"build",value:function a(){return this._componentTree.hasRoot()?this._build([this._componentTree.rootId],{}):{}}},{key:"_build",value:function d(a,b){var c=this;return a.forEach(function(a){var d=c._componentTree.get(a),e=d.childIds,f=d.keyMap;f&&Object.keys(f).forEach(function(a){normalizeActionDefinition(f,a,b);}),c._build(e,b);}),b}}]),a}();

var KeyEventManager=function(){function a(){var b=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{};_classCallCheck(this,a);var c=Configuration.option("logLevel");this.logger=b.logger||new Logger(c),this.focusOnlyEventStrategy=new FocusOnlyKeyEventStrategy({configuration:b,logLevel:c},this),this.globalEventStrategy=new GlobalKeyEventStrategy({configuration:b,logLevel:c},this),this.mountedComponentsCount=0,this._blurHandler=this._clearKeyHistory.bind(this);}return _createClass(a,null,[{key:"getInstance",value:function c(){var b=0<arguments.length&&void 0!==arguments[0]?arguments[0]:{};return lazyLoadAttribute(this,"instance",function(){return new a(b)})}},{key:"getFocusOnlyEventStrategy",value:function a(){return this.getInstance().focusOnlyEventStrategy}},{key:"getGlobalEventStrategy",value:function a(){return this.getInstance().globalEventStrategy}}]),_createClass(a,[{key:"registerComponentMount",value:function c(a,b){return this._incrementComponentCount(),this.focusOnlyEventStrategy.registerComponentMount(a,b)}},{key:"registerComponentUnmount",value:function a(){this._decrementComponentCount();}},{key:"_incrementComponentCount",value:function b(){var a=this.mountedComponentsCount;this.mountedComponentsCount+=1,0===a&&1===this.mountedComponentsCount&&window.addEventListener("blur",this._blurHandler);}},{key:"_decrementComponentCount",value:function b(){var a=this.mountedComponentsCount;this.mountedComponentsCount-=1,1===a&&0===this.mountedComponentsCount&&window.removeEventListener("blur",this._blurHandler);}},{key:"_clearKeyHistory",value:function a(){this.focusOnlyEventStrategy.resetKeyHistory({force:!0}),this.globalEventStrategy.resetKeyHistory({force:!0});}},{key:"registerGlobalComponentUnmount",value:function a(){this._decrementComponentCount();}},{key:"registerGlobalComponentMount",value:function c(a,b){return this._incrementComponentCount(),this.globalEventStrategy.registerComponentMount(a,b)}},{key:"addKeyCombinationListener",value:function b(a){return this.globalEventStrategy.addKeyCombinationListener(a)}},{key:"ignoreEvent",value:function b(a){this.focusOnlyEventStrategy.eventPropagator.ignoreEvent(a);}},{key:"observeIgnoredEvents",value:function b(a){this.focusOnlyEventStrategy.eventPropagator.observeIgnoredEvents(a);}},{key:"closeHangingKeyCombination",value:function c(a,b){this.focusOnlyEventStrategy.closeHangingKeyCombination(a,b);}},{key:"reactAppHistoryWithEvent",value:function d(a,b){var c=this.focusOnlyEventStrategy.eventPropagator.previousPropagation;return c.isForKey(a)&&c.isForEventType(b)?c.isHandled()?EventResponse.handled:c.isIgnoringEvent()?EventResponse.ignored:EventResponse.seen:EventResponse.unseen}},{key:"simulatePendingKeyPressEvents",value:function a(){this.focusOnlyEventStrategy.simulatePendingKeyPressEvents();}},{key:"simulatePendingKeyUpEvents",value:function a(){this.focusOnlyEventStrategy.simulatePendingKeyUpEvents();}},{key:"isGlobalListenersBound",value:function a(){return this.globalEventStrategy.isListenersBound()}},{key:"applicationKeyMap",get:function a(){return [this.globalEventStrategy,this.focusOnlyEventStrategy].reduce(function(a,b){var c=new ApplicationKeyMapBuilder(b.componentTree),d=c.build();return _objectSpread({},a,d)},{})}}]),a}();

function wrapPropInFunction(a,b){return "function"==typeof a?function(c){a(c),b(c);}:b}function getComponentOptions(){return {defaultKeyEvent:Configuration.option("defaultKeyEvent")}}function getEventOptions(){return {ignoreEventsCondition:Configuration.option("ignoreEventsCondition")}}var FocusOnlyComponentManager=function(){function a(b,c){var d=c.keyMap;_classCallCheck(this,a),this._hotKeysOptions=b,this.id=KeyEventManager.getFocusOnlyEventStrategy().registerKeyMap(d),this.childContext={hotKeysParentId:this.id},this._focusTreeIds=[];}return _createClass(a,[{key:"getComponentProps",value:function d(a){var b=this,c={onFocus:wrapPropInFunction(a.onFocus,function(){return b.enableHotKeys(a)}),onBlur:wrapPropInFunction(a.onBlur,function(){return b.disableHotKeys(a)}),tabIndex:Configuration.option("defaultTabIndex")};return this._shouldBindKeyListeners(a)&&(c.onKeyDown=function(a){return b._delegateEventToManager(a,"handleKeyDown")},c.onKeyPress=function(a){return b._delegateEventToManager(a,"handleKeyPress")},c.onKeyUp=function(a){return b._delegateEventToManager(a,"handleKeyUp")}),c}},{key:"_shouldBindKeyListeners",value:function c(a){var b=this._getKeyMap(a);return !isEmpty(b)||a.root}},{key:"addHotKeys",value:function c(a){var b=KeyEventManager.getInstance();b.registerComponentMount(this.id,a);}},{key:"enableHotKeys",value:function c(a){a.onFocus&&a.onFocus.apply(a,arguments);var b=KeyEventManager.getFocusOnlyEventStrategy().enableHotKeys(this.id,this._getKeyMap(a),this._getHandlers(a),getComponentOptions());isUndefined(b)||this._focusTreeIdsPush(b),this._setFocused(!0);}},{key:"updateHotKeys",value:function e(a){var b=KeyEventManager.getFocusOnlyEventStrategy();if(b.reregisterKeyMap(this.id,a.keyMap),this._componentIsFocused()&&(a.allowChanges||!Configuration.option("ignoreKeymapAndHandlerChangesByDefault"))){var c=a.keyMap,d=a.handlers;b.updateEnabledHotKeys(this.focusTreeId,this.id,c,d,getComponentOptions());}}},{key:"disableHotKeys",value:function c(a){a.onBlur&&a.onBlur.apply(a,arguments);var b=KeyEventManager.getFocusOnlyEventStrategy().disableHotKeys(this.focusTreeId,this.id);b||this._focusTreeIdsShift(),this._setFocused(!1);}},{key:"removeKeyMap",value:function c(a){var b=KeyEventManager.getFocusOnlyEventStrategy();b.deregisterKeyMap(this.id),KeyEventManager.getInstance().registerComponentUnmount(),this.disableHotKeys(a);}},{key:"_componentIsFocused",value:function a(){return !0===this._focused}},{key:"_focusTreeIdsPush",value:function b(a){this._focusTreeIds.push(a);}},{key:"_focusTreeIdsShift",value:function a(){this._focusTreeIds.shift();}},{key:"_setFocused",value:function b(a){this._focused=a;}},{key:"_delegateEventToManager",value:function d(a,b){var c=KeyEventManager.getFocusOnlyEventStrategy()[b](a,this.focusTreeId,this.id,getEventOptions());c&&this._focusTreeIdsShift();}},{key:"_mergeWithOptions",value:function c(a,b){return _objectSpread({},this._hotKeysOptions[a]||{},b[a]||{})}},{key:"_getHandlers",value:function b(a){return this._mergeWithOptions("handlers",a)}},{key:"_getKeyMap",value:function b(a){return this._mergeWithOptions("keyMap",a)}},{key:"focusTreeId",get:function a(){return this._focusTreeIds[0]}}]),a}();

var propTypes={keyMap:PropTypes.object,handlers:PropTypes.object,onFocus:PropTypes.func,onBlur:PropTypes.func,allowChanges:PropTypes.bool,root:PropTypes.bool};function provideWithContext(a){return backwardsCompatibleContext(a,{deprecatedAPI:{contextTypes:{hotKeysParentId:PropTypes.number},childContextTypes:{hotKeysParentId:PropTypes.number}},newAPI:{contextType:{hotKeysParentId:void 0}}})}function componentFactory(a){var b,c,d=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{};return c=b=function(b){function c(a){var b;return _classCallCheck(this,c),b=_possibleConstructorReturn(this,_getPrototypeOf(c).call(this,a)),b._manager=new FocusOnlyComponentManager(d,a),b}return _inherits(c,b),_createClass(c,[{key:"componentDidMount",value:function a(){this._manager.addHotKeys(this.context.hotKeysParentId);}},{key:"componentDidUpdate",value:function a(){this._manager.updateHotKeys(this.props);}},{key:"componentWillUnmount",value:function a(){this._manager.removeKeyMap(this.props);}},{key:"render",value:function h(){var b=this.props,c=b.keyMap,d=b.handlers,e=b.allowChanges,f=b.root,g=_objectWithoutProperties(b,["keyMap","handlers","allowChanges","root"]);return React.createElement(a,_extends({hotKeys:this._manager.getComponentProps(this.props)},g))}}]),c}(PureComponent),_defineProperty(b,"propTypes",propTypes),c}function withHotKeys(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{};return provideWithContext(componentFactory(a,b))}

var HotKeysWrapper=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"render",value:function g(){var a=this.props,b=a.hotKeys,c=a.innerRef,d=a.component,e=_objectWithoutProperties(a,["hotKeys","innerRef","component"]),f=d||Configuration.option("defaultComponent");return React.createElement(f,_objectSpread({},b,{ref:c},e))}}]),b}(Component),HotKeys=withHotKeys(HotKeysWrapper);HotKeys.propTypes={innerRef:PropTypes.oneOfType([PropTypes.object,PropTypes.func])};

var GlobalComponentManager=function(){function a(b){_classCallCheck(this,a),this.id=KeyEventManager.getGlobalEventStrategy().registerKeyMap(b),this.childContext={globalHotKeysParentId:this.id};}return _createClass(a,null,[{key:"_getComponentOptions",value:function a(){return {defaultKeyEvent:Configuration.option("defaultKeyEvent")}}},{key:"_getEventOptions",value:function a(){return {ignoreEventsCondition:Configuration.option("ignoreEventsCondition")}}}]),_createClass(a,[{key:"addHotKeys",value:function g(b,c){var d=b.keyMap,e=b.handlers,f=c.globalHotKeysParentId;KeyEventManager.getInstance().registerGlobalComponentMount(this.id,f),KeyEventManager.getGlobalEventStrategy().enableHotKeys(this.id,d,e,a._getComponentOptions(),a._getEventOptions());}},{key:"updateHotKeys",value:function f(b){var c=KeyEventManager.getGlobalEventStrategy();if(c.reregisterKeyMap(this.id,b.keyMap),b.allowChanges||!Configuration.option("ignoreKeymapAndHandlerChangesByDefault")){var d=b.keyMap,e=b.handlers;c.updateEnabledHotKeys(this.id,d,e,a._getComponentOptions(),a._getEventOptions());}}},{key:"removeKeyMap",value:function b(){var a=KeyEventManager.getGlobalEventStrategy();a.deregisterKeyMap(this.id),a.disableHotKeys(this.id),KeyEventManager.getInstance().registerGlobalComponentUnmount();}}]),a}();

var GlobalHotKeys=function(a){function b(a){var c;return _classCallCheck(this,b),c=_possibleConstructorReturn(this,_getPrototypeOf(b).call(this,a)),c._manager=new GlobalComponentManager(a.keyMap),c}return _inherits(b,a),_createClass(b,[{key:"render",value:function a(){return this.props.children||null}},{key:"componentDidMount",value:function a(){this._manager.addHotKeys(this.props,this.context);}},{key:"componentDidUpdate",value:function a(){this._manager.updateHotKeys(this.props);}},{key:"componentWillUnmount",value:function a(){this._manager.removeKeyMap();}}]),b}(Component);_defineProperty(GlobalHotKeys,"propTypes",{keyMap:PropTypes.object,handlers:PropTypes.object,allowChanges:PropTypes.bool});var GlobalHotKeys$1 = backwardsCompatibleContext(GlobalHotKeys,{deprecatedAPI:{contextTypes:{globalHotKeysParentId:PropTypes.number},childContextTypes:{globalHotKeysParentId:PropTypes.number}},newAPI:{contextType:{globalHotKeysParentId:void 0}}});

var HotKeysIgnoreOverrideManager=function(){function a(b){_classCallCheck(this,a),this._handleKeyEvent=this._handleKeyEvent.bind(this),this._eventManagerMethod=b;}return _createClass(a,[{key:"getComponentProps",value:function c(a){var b=this;return {onKeyDown:this._handleKeyEvent,onKeyPress:this._handleKeyEvent,onKeyUp:this._handleKeyEvent,onFocus:function c(){return b._reloadDictionaries(a())}}}},{key:"_reloadDictionaries",value:function d(a){var b=a.only,c=a.except;this._onlyDict=keyDictionary(b),this._exceptDict=keyDictionary(c);}},{key:"_handleKeyEvent",value:function b(a){this._shouldIgnoreEvent(a)&&KeyEventManager.getInstance()[this._eventManagerMethod](a);}},{key:"_shouldIgnoreEvent",value:function c(a){var b=a.key;return isEmpty(this._onlyDict)?!!isEmpty(this._exceptDict)||!hasKey(this._exceptDict,b):isEmpty(this._exceptDict)?hasKey(this._onlyDict,b):hasKey(this._onlyDict,b)&&!hasKey(this._exceptDict,b)}}]),a}();function keyDictionary(a){return arrayFrom(a).reduce(function(a,b){var c=standardizeKeyName(b);if(!isValidKey(c))throw new InvalidKeyNameError(b);return dictionaryFrom([resolveAltShiftedAlias,resolveUnaltShiftedAlias,resolveShiftedAlias,resolveUnshiftedAlias,resolveAltedAlias,resolveUnaltedAlias],!0,function(a){return a(c)},a),a},{})}

var propTypes$1={only:PropTypes.oneOfType([PropTypes.string,PropTypes.arrayOf(PropTypes.string)]),except:PropTypes.oneOfType([PropTypes.string,PropTypes.arrayOf(PropTypes.string)])};function withHotKeysIgnoreOverride(a){var b,c,d=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{only:[],except:[]},e=2<arguments.length?arguments[2]:void 0;return c=b=function(b){function c(a){var b;return _classCallCheck(this,c),b=_possibleConstructorReturn(this,_getPrototypeOf(c).call(this,a)),b._manager=new HotKeysIgnoreOverrideManager(e),b}return _inherits(c,b),_createClass(c,[{key:"render",value:function g(){var b=this,c=this.props,d=c.only,e=c.except,f=_objectWithoutProperties(c,["only","except"]);return React.createElement(a,_extends({hotKeys:this._manager.getComponentProps(function(){return b.props})},f))}}]),c}(PureComponent),_defineProperty(b,"propTypes",propTypes$1),_defineProperty(b,"defaultProps",d),c}

function overrideComponent(a){var b,c;return c=b=function(a){function b(){return _classCallCheck(this,b),_possibleConstructorReturn(this,_getPrototypeOf(b).apply(this,arguments))}return _inherits(b,a),_createClass(b,[{key:"render",value:function e(){var a=this.props,b=a.hotKeys,c=_objectWithoutProperties(a,["hotKeys"]),d=c.component||Configuration.option("defaultComponent");return React.createElement(d,_objectSpread({},b,c))}}]),b}(Component),_defineProperty(b,"displayName",a),c}

var IgnoreKeys = withHotKeysIgnoreOverride(overrideComponent("IgnoreKeys"),{},"ignoreEvent");

var ObserveKeys = withHotKeysIgnoreOverride(overrideComponent("ObserveKeys"),{},"observeIgnoredEvents");

function withIgnoreKeys(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{only:[],except:[]};return withHotKeysIgnoreOverride(a,b,"ignoreEvent")}

function withObserveKeys(a){var b=1<arguments.length&&arguments[1]!==void 0?arguments[1]:{only:[],except:[]};return withHotKeysIgnoreOverride(a,b,"observeIgnoredEvents")}

function configure(){var a=0<arguments.length&&arguments[0]!==void 0?arguments[0]:{};Configuration.init(a);}

function getApplicationKeyMap(){return KeyEventManager.getInstance().applicationKeyMap}

function recordKeyCombination(a){var b=KeyEventManager.getInstance();return b.addKeyCombinationListener(a)}

export { HotKeys, GlobalHotKeys$1 as GlobalHotKeys, IgnoreKeys, ObserveKeys, withHotKeys, withIgnoreKeys, withObserveKeys, configure, getApplicationKeyMap, recordKeyCombination };
